<?php

/*
Copyright 2008 iThemes (email: support@ithemes.com)

Written by Chris Jean
Version 1.1.2

Version History
	1.0.1 - 2008-09-30
	1.0.2 - 2008-10-03
	1.0.3 - 2008-11-02
		Fixed file path to url conversion issues
	1.0.4 - 2008-11-03
		Added check for WP_Error on image resize
	1.0.5 - 2008-11-07
		Fixed bug in resizeImage function
	1.0.6 - 2008-11-08
		Fixed Windows compatibility bug in resizeImage function
	1.0.7 - 2008-11-08
		Sped up _getResizedImageFilePath code
	1.0.8 - 2008-11-10
		Cleaned up output
	1.1.0 - 2008-11-11
		Replaced WP resize calls with internal code to provide expandable resized images
	1.1.1 - 2008-11-13
		Compatibility fixes for both Windows and PHP4
		Added backwards compatibility calls for use in older themes
	1.1.2 - 2008-11-25
		Switched references to get_option( 'home' ) to get_option( 'siteurl' )
*/


if ( !class_exists( 'iThemesFileUtility' ) ) {
	class iThemesFileUtility {
		// For backwards compatibility
		function uploadFile( $file_id ) {
			return iThemesFileUtility::upload_file( $file_id );
		}
		
		// Customized media_handle_upload() from 2.6.2 wp-admin/includes/media.php
		function upload_file( $file_id ) {
			$overrides = array( 'test_form' => false );
			$file = wp_handle_upload( $_FILES[$file_id], $overrides );
			
			if ( isset( $file['error'] ) )
				return new WP_Error( 'upload_error', $file['error'] );
			
			$url = $file['url'];
			$type = $file['type'];
			$file = $file['file'];
			$title = preg_replace( '/\.[^.]+$/', '', basename( $file ) );
			$content = '';
			
			// use image exif/iptc data for title and caption defaults if possible
			if ( $image_meta = @wp_read_image_metadata( $file ) ) {
				if ( trim( $image_meta['title'] ) )
					$title = $image_meta['title'];
				if ( trim( $image_meta['caption'] ) )
					$content = $image_meta['caption'];
			}
			
			// Construct the attachment array
			$attachment = array(
				'post_mime_type' => $type,
				'guid' => $url,
				'post_title' => $title,
				'post_content' => $content
			);
			
			// Save the data
			$id = wp_insert_attachment( $attachment, $file );
			if ( !is_wp_error( $id ) ) {
				wp_update_attachment_metadata( $id, wp_generate_attachment_metadata( $id, $file ) );
			}
			
			
			$data = array(
				'id'		=> $id,
				'url'		=> $url,
				'type'		=> $type,
				'file'		=> $file,
				'title'		=> $title,
				'caption'	=> $content
			);
			
			return $data;
		}
		
		// For backwards compatibility. Note how the default crop changed from false to true in the new version
		function resizeImage( $file, $max_w, $max_h, $crop = false, $suffix = null, $dest_path = null, $jpeg_quality = 90 ) {
			return iThemesFileUtility::resize_image( $file, $max_w, $max_h, $crop, $suffix, $dest_path, $jpeg_quality );
		}
		
		// Customized resize_image() from 2.6.3 wp-admin/includes/media.php (added resized image caching)
		function resize_image( $file, $max_w, $max_h, $crop = true, $suffix = null, $dest_path = null, $jpeg_quality = 90 ) {
			require_once( ABSPATH . 'wp-admin/includes/image.php' );
			
			$image = wp_load_image( $file );
			if ( ! is_resource( $image ) )
				return new WP_Error( 'error_loading_image', $image );
			
			list( $orig_w, $orig_h, $orig_type ) = getimagesize( $file );
			$dims = iThemesFileUtility::_image_resize_dimensions( $orig_w, $orig_h, $max_w, $max_h, $crop );
			if ( ! $dims )
				return new WP_Error( 'error_resizing_image', "Could not resize image" );
			list( $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h ) = $dims;
			
			// $suffix will be appended to the destination filename, just before the extension
			if ( ! $suffix )
				$suffix = "{$dst_w}x{$dst_h}";
			
			$info = pathinfo( $file );
			$dir = $info['dirname'];
			$ext = $info['extension'];
			$name = basename( $file, ".{$ext}" );
			
			if ( ! is_null( $dest_path ) and $_dest_path = realpath( $dest_path ) )
				$dir = $_dest_path;
			$destfilename = "{$dir}/{$name}-{$suffix}.{$ext}";
			
			
			if ( file_exists( $destfilename ) ) {
				if ( filemtime( $file ) > filemtime( $destfilename ) )
					unlink( $destfilename );
				else
					return array( 'file' => $destfilename, 'url' => iThemesFileUtility::getURLFromFile( $destfilename ) );
			}
			
			
			$newimage = imagecreatetruecolor( $dst_w, $dst_h );
			
			// preserve PNG transparency
			if ( IMAGETYPE_PNG == $orig_type && function_exists( 'imagealphablending' ) && function_exists( 'imagesavealpha' ) ) {
				imagealphablending( $newimage, false );
				imagesavealpha( $newimage, true );
			}
			
			imagecopyresampled( $newimage, $image, $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h );
			
			// we don't need the original in memory anymore
			imagedestroy( $image );
			
			if ( $orig_type == IMAGETYPE_GIF ) {
				if ( ! imagegif( $newimage, $destfilename ) )
					return new WP_Error( 'resize_path_invalid', __( 'Resize path invalid' ) );
			}
			elseif ( $orig_type == IMAGETYPE_PNG ) {
				if ( ! imagepng( $newimage, $destfilename ) )
					return new WP_Error( 'resize_path_invalid', __( 'Resize path invalid' ) );
			}
			else {
				// all other formats are converted to jpg
				$destfilename = "{$dir}/{$name}-{$suffix}.jpg";
				if ( ! imagejpeg( $newimage, $destfilename, apply_filters( 'jpeg_quality', $jpeg_quality ) ) )
					return new WP_Error( 'resize_path_invalid', __( 'Resize path invalid' ) );
			}
			
			imagedestroy( $newimage );
			
			// Set correct file permissions
			$stat = stat( dirname( $destfilename ) );
			$perms = $stat['mode'] & 0000666; //same permissions as parent folder, strip off the executable bits
			@ chmod( $destfilename, $perms );
			
			
			return array( 'file' => $destfilename, 'url' => iThemesFileUtility::getURLFromFile( $destfilename ) );
		}
		
		// Customized image_resize_dimensions() from 2.6.3 wp-admin/includes/media.php (cheanged to resize to fill on crop)
		function _image_resize_dimensions( $orig_w, $orig_h, $dest_w, $dest_h, $crop = false ) {
			if ( ( $orig_w <= 0 ) || ( $orig_h <= 0 ) )
				return new WP_Error ( 'error_resizing_image', "Supplied invalid original dimensions ($orig_w, $orig_h)" );
			if ( ( $dest_w <= 0 ) && ( $dest_h <= 0 ) )
				return new WP_Error ( 'error_resizing_image', "Supplied invalid destination dimentions ($dest_w, $dest_h)" );
			
			
			$new_w = $dest_w;
			$new_h = $dest_h;
			
			$s_x = 0;
			$s_y = 0;
			
			$crop_w = $orig_w;
			$crop_h = $orig_h;
			
			
			if ( $crop ) {
				$cur_ratio = $orig_w / $orig_h;
				$new_ratio = $dest_w / $dest_h;
				
				if ( $cur_ratio > $new_ratio ) {
					$crop_w = floor( $orig_w / ( ( $dest_h / $orig_h ) / ( $dest_w / $orig_w ) ) );
					$s_x = floor( ( $orig_w - $crop_w ) / 2 );
				}
				elseif ( $new_ratio > $cur_ratio ) {
					$crop_h = floor( $orig_h / ( ( $dest_w / $orig_w ) / ( $dest_h / $orig_h ) ) );
					$s_y = floor( ( $orig_h - $crop_h ) / 2 );
				}
			}
			else
				list( $new_w, $new_h ) = wp_constrain_dimensions( $orig_w, $orig_h, $dest_w, $dest_h );
			
			
			return array( 0, 0, $s_x, $s_y, $new_w, $new_h, $crop_w, $crop_h );
		}
		
		function getURLFromFile( $file ) {
			return get_option( 'siteurl' ) . str_replace( '\\', '/', str_replace( rtrim( ABSPATH, '\\\/' ), '', $file ) );
		}
	}
}

?>
